__author__ = 'alex'

from parts.stylesheets import StylesheetsBuilder
from parts.deps import DepsBuilder
from parts.linter import Linter
from parts.js import JSBuilder
from bootstrap import bootstrap
from parts.templates import TemplatesBuilder
from server import serve